#include <stdio.h>
#include <windows.h>
#include <vfw.h>
#include <math.h>
#include "cbw.h"
#include "vahm32.h"
#include "main.h"

/* variable pour les capteurs US */
HANDLE g_hPort6811; /* Handle de la liaison srie */

/* variables pour la librairie universelle */
float RevNum = (float)( 3.3 ); /* Revision de la librairie universelle */
int CNA_BoardNum = 0; /* Numro de la carte CNA : configur avec InstaCal */
int ODOM_BoardNum = 1; /* Numro de la carte Codeur Incrmentale :
					      configur avec InstaCal */
float DLLRevNum; /* Revision de la DLL */
float VXDRevNum; /* Revision du VXD */
HGLOBAL CNA_MemBuffer;

/* variables pour la vision (video for windows) */
HWND g_hCapture; /* Handle de la fentre d'acquisition */
CAPDRIVERCAPS g_sDriverCaps; /* Caractristique du periph. d'acq. vido */
unsigned int g_uiIDDriverVideo; /* Identificateur du periph d'acq. vido */
BITMAPINFO g_bmiImage; /* Informations sur l'image acquise, taille, nbpp, ...*/
BYTE *g_bImage; /* Image en niveau de gris */


/*****************************************************************************
 * Point d'entre de la DLL
 *****************************************************************************/
BOOL WINAPI DllMain( HINSTANCE hInstDLL, DWORD fdwReason, LPVOID lpReserved )
{
	switch( fdwReason )
	{
	case DLL_PROCESS_ATTACH:
		/* la DLL est mappe dans l'espace d'adressage du processus */
		SetUnhandledExceptionFilter( (LPTOP_LEVEL_EXCEPTION_FILTER)( exceptionhandler ) );
		break;
	case DLL_THREAD_ATTACH:
		/* Un thread est cr */
		break;
	case DLL_THREAD_DETACH:
		/* Un thread quitte l'application correctement */
		break;
	case DLL_PROCESS_DETACH:
		/* La DLL est dmappe de l'espace d'adressage du processus */
		SetUnhandledExceptionFilter( (LPTOP_LEVEL_EXCEPTION_FILTER)( NULL ) );
		break;
	}
	return TRUE; /* uniquement utilis pour DLL_PROCESS_ATTACH */
}

/*****************************************************************************
 * Procdure excute lors d'un arrt anormal de l'application 
 *****************************************************************************/
LONG exceptionhandler( LPEXCEPTION_POINTERS arg )
{
	/* en cas de terminaison anormale du programme, on ferme le VAHM */
	VAHM32_CloseVAHM();
	return EXCEPTION_CONTINUE_SEARCH;
}

/*****************************************************************************
 * Simple boite de message pour afficher la version de la DLL
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_AProposDe( )
{
	char texte[128];

   	sprintf( texte, 
		"VAHM32.DLL : Ver 0.0.3\n17/09/2001\nGestion des US, Moteurs, Vision\n" );
	MessageBoxEx( NULL, texte, "vahm32",
		MB_OK | MB_ICONINFORMATION, LANG_FRENCH );

	return VAHM32_OK;
}

/*****************************************************************************
 * Initialisation du VAHM
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_InitVAHM( )
{
	int nCodeRetour;
//	unsigned int idxDriver;

	/* initialisation des capteurs US */
	/* ouverture de la liaison srie avec la carte 68HC11 */
	g_hPort6811 = OuvrirLiaisonSerie( "COM2" );
	if( g_hPort6811 != NULL )
	{
		/* dmarrage de la boucle de lecture de capteurs */
		Demarrer68HC11( g_hPort6811 );
	}
	else
		return VAHM32_ECHEC_LIAISON68HC11;

	/* initialisation de la librairie universelle */
	if( Init_UL() != VAHM32_OK )
		return VAHM32_UNILIB_ERREUR;

	/* Mise  l'arret  des moteurs */
	VAHM32_EcritureTension( (float)( 2.54 ), (float)( 2.54 ) );

	/* Initialisation de l'odomtrie */
	nCodeRetour = InitCompteursOdom( );
	if( nCodeRetour != NOERRORS )
		return VAHM32_ODOM_ERREUR;
	
	/* initialisation de l'acquisition vido */
	/* cration de la fentre d'acquisition */
//	g_hCapture = capCreateCaptureWindow( "Vido", WS_OVERLAPPEDWINDOW|WS_VISIBLE,
//		0, 0, 160, 120, NULL, 0 );
	/* masquage de la fentre d'aquisition vido */
//	ShowWindow( g_hCapture, SW_HIDE );
	/* Initialisation des procdures appele lors d'acqu. ou de problme */
//	capSetCallbackOnFrame( g_hCapture, VideoCallbackProc );
//	capSetCallbackOnError( g_hCapture, ErreurCallbackProc );
	/* Rcupration des caractristiques du priphrique d'acq. vido */
//	capDriverGetCaps( g_hCapture, &g_sDriverCaps, sizeof( g_sDriverCaps ) );
	/* Connextion au periph d'acquisition */
//	idxDriver = 0;
//	while( ( capDriverConnect( g_hCapture, idxDriver ) == FALSE ) &&
//		( idxDriver <= 9 ) )
//	{
//		idxDriver++;
//	}
//	if( idxDriver > 9 )
//	{
//		DestroyWindow( g_hCapture );
//		MessageBox( NULL, 
//			"Impossible de Connecter le Priphrique de capture vido",
//			"Vahm32",
//			MB_OK|MB_ICONINFORMATION );
//	}
//	else
//	{
//		g_uiIDDriverVideo = idxDriver;
//	}
	/* dfinition des caractristiques de l'acquisition */
//	g_bmiImage.bmiHeader.biBitCount = 24; /* nombre de bits par pixels : 24 = 8r+8v+8b */
//	g_bmiImage.bmiHeader.biClrImportant = 0; /* toutes les couleurs sont importantes */
//	g_bmiImage.bmiHeader.biClrUsed = 0; /* toutes les couleurs sont utilises */
//	g_bmiImage.bmiHeader.biCompression = BI_RGB; /* pas de compression */
//	g_bmiImage.bmiHeader.biHeight = 288; /* hauteur de l'image */
//	g_bmiImage.bmiHeader.biWidth = 384; /* largeur de l'image */
//	g_bmiImage.bmiHeader.biPlanes = 1; /* 1 plan et 24 bits donc les donnes sont ranges comme : rvb,rvb,rvb,rvb.... */
//	g_bmiImage.bmiHeader.biSize = sizeof( BITMAPINFOHEADER ); /* taille de la structure */
//	g_bmiImage.bmiHeader.biSizeImage = 
//		g_bmiImage.bmiHeader.biHeight *
//		g_bmiImage.bmiHeader.biWidth * 
//		g_bmiImage.bmiHeader.biBitCount / 8; /* taille de l'image en octets */
//	g_bmiImage.bmiHeader.biXPelsPerMeter = 160000; /* rsolution horizontale */
//	g_bmiImage.bmiHeader.biYPelsPerMeter = 160000; /* rsolution verticale */

	/* initialisation des caractristique de l'aquisition */
//	capSetVideoFormat( g_hCapture, &g_bmiImage, sizeof( BITMAPINFO ) );

	/* allocation d'un espace mmoire ou stocker l'image en NVG */
//	g_bImage = (BYTE*)malloc( sizeof( BYTE ) * g_bmiImage.bmiHeader.biHeight *
//		g_bmiImage.bmiHeader.biWidth );

	return VAHM32_OK;
}

/*****************************************************************************
 * Arret du VAHM
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_CloseVAHM( )
{
	if( g_hPort6811 != NULL )
	{
		/* fermeture de la liaison srie -> 68HC11 */
		FermerLiaisonSerie( g_hPort6811 );
	}
	/* Mise  l'arret  des moteurs */
	VAHM32_EcritureTension( (float)( 2.54 ), (float)( 2.54 ) );

	/* destruction de l'image tempon */
	if( g_bImage != NULL )
		free( g_bImage );

	/* fermeture de liaison avec le periph d'acq. vido */
	if( g_hCapture != NULL )
	{
		capSetCallbackOnFrame( g_hCapture, NULL );
		capSetCallbackOnError( g_hCapture, NULL );
		capDriverDisconnect( g_hCapture );
		DestroyWindow( g_hCapture );
	}

	return VAHM32_OK;
}

/*****************************************************************************
 * Lecture des capteurs ultrasons
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_LectureUS( float *BufferUS )
{
	if( g_hPort6811 != NULL )
	{
		/* lecture du buffer de la liaison srie */
		LectureCapteursUS( g_hPort6811, BufferUS );
	}
	else
		return VAHM32_LECTURE_US_IMPOSSIBLE;

	return VAHM32_OK;
}

/*****************************************************************************
 * Lecture des capteurs ultrasons
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_AffichageUS( HWND hwnd, 
									int Xorg, int Yorg, 
									float *BufferUS )
{
	HDC hdc;
	int i;
	char Texte[ 80 ];
	int nTailleTexte;

	/* rcupration du contexte de priphrique */
	hdc = GetDC( hwnd );
	
	/* affichage de la valeur de l'intrupteur de commande */
	nTailleTexte = sprintf( Texte, "Capteur TOR = %4.0f", BufferUS[ 0 ] );
	TextOut( hdc, Xorg, Yorg, Texte, nTailleTexte ); 
	
	/* affichage des valeurs US */
	for( i = 0; i < VAHM32_NBRCAPTSUS; i++ )
	{
		nTailleTexte = sprintf( Texte, "Capteur[ %#2d ] = %4.2f", i, BufferUS[ i ] );
		TextOut( hdc, Xorg, Yorg + 20 * i, Texte, nTailleTexte ); 
	}
	
	return VAHM32_OK;
}

/*****************************************************************************
 * Acquisition d'une image
 *****************************************************************************/
DWORD EXPORTVAHM32 VAHM32_CaptureImage( )
{
	capGrabFrame( g_hCapture );
	
	return (DWORD)( g_bImage );
}

/*****************************************************************************
 * Sauvegarde au format PGM
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_SauveImgPGM( BYTE *image, char* nomFichier )
{
	FILE* pFichier;
	int ierr;
	
	pFichier = fopen( nomFichier, "wb" );

	if( pFichier == NULL )
		return VAHM32_ERREUR;

	fprintf( pFichier, "P5\n" );
  	fprintf( pFichier, "# VAHM - LASC\n");
	fprintf( pFichier, "%d %d\n", 
		g_bmiImage.bmiHeader.biWidth, g_bmiImage.bmiHeader.biHeight );
  	fprintf( pFichier, "255\n" );

  	ierr = fwrite( image, 1, 
		g_bmiImage.bmiHeader.biWidth * g_bmiImage.bmiHeader.biHeight, 
		pFichier );
  	if( ierr != (int)( g_bmiImage.bmiHeader.biWidth *
		g_bmiImage.bmiHeader.biHeight ) )
	{
		fclose(pFichier);
		return VAHM32_ERREUR;
	}

	fclose(pFichier);	

	return VAHM32_OK;
}

/****************************************************************************
 * Initialisation de la librairie universelle
 ****************************************************************************/
int Init_UL( void )
{
	int nResultat;
	/* Initialisation de la librairie universelle avec le numro de rvision
       utilis pour crire le programme. Premire fonction de la L.U. qui doit
       tre appele par le programme */
	nResultat = cbDeclareRevision( &RevNum );
	if( nResultat != 0 ) /* Problme d'initialisation */
		return VAHM32_PROBLEME;
	else
		return VAHM32_OK;
}

/*****************************************************************************
 * Ecriture des tension sur le bus DX
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_EcritureTension( float v1, float v2 )
{
	/* Allocation d'un buffer mmoire pour les quatres tension */
	CNA_MemBuffer = AlloueTemponMemoire( 4 );
	
	/* Ecriture des tensions dans les buffer et validation de la convertion */
	EcrireTension( CNA_MemBuffer, v1, v2 );
	
	/* Dsalocation Mmoire */
	DesalloueTemponMemoire( CNA_MemBuffer );

	return VAHM32_OK;
}

/*****************************************************************************
 * Lecture des codeurs incrmentaux
 *****************************************************************************/
/*****************************************************************************
 *        *                GAUCHE : Voie 2
 *       * *               DROITE : Voie 1
 *      * * *
 *     *  *  *             RESPECTER L'ORDRE DES CONNEXIONS !!!!   
 *    *   *   *
 *   *         *
 *  *     *     *
 * ***************
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_LireCodeurs( long *ulCntDrt, long *ulCntGch )
{
	/* Lecture du premier Compteur */
	*ulCntDrt = ( long )( - LireCompteurOdom( 1 ) );
	/* Lecture du second Compteur */
	*ulCntGch = ( long )( LireCompteurOdom( 2 ) );

	return VAHM32_OK;
}

/*****************************************************************************
 * Remise  Zro des compteurs
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_RAZCodeurs( )
{
	return RAZCompteursOdom( );
}

/*****************************************************************************
 * Mise  une valeur des compteurs
 *****************************************************************************/
int EXPORTVAHM32 VAHM32_SETCodeurs( long ctn1, long ctn2 )
{
	return SETCompteursOdom( ctn1, ctn2 );
}
